SystemOrganization addCategory: #'Glamour-Tests'!

GLMPane subclass: #GLMTestPane
	instanceVariableNames: 'seenTransmissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMTestPane classSide methodsFor: 'private' stamp: ' 25/2/09 23:39'!
portClass 
	 
	^GLMTestPort! !

TestCase subclass: #GLMBundleTransmissionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMBundleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPorts 
	"Tests that the origins are set up properly and that their order is retained." 
	 
	| transmission | 
	transmission := GLMBundleTransmission new. 
	transmission 
		addActiveOrigin: 
			((GLMSimplePort new) 
				name: #port1; 
				value: 1). 
	transmission 
		addPassiveOrigin: 
			((GLMSimplePort new) 
				name: #port2; 
				value: 2). 
	transmission 
		addActiveOrigin: 
			((GLMSimplePort new) 
				name: #port3; 
				value: 3). 
	transmission 
		addPassiveOrigin: 
			((GLMSimplePort new) 
				name: #port4; 
				value: 4). 
	transmission destination: (GLMSimplePort new name: #port5). 
	self assert: transmission bundleOrigins size = 4. 
	transmission bundleOrigins 
		do: [:each | self assert: (each isKindOf: GLMBundleOrigin)]. 
	self assert: (transmission bundleOrigins at: 1) port name = #port1. 
	self assert: (transmission bundleOrigins at: 2) port name = #port2. 
	self assert: (transmission bundleOrigins at: 3) port name = #port3. 
	self assert: (transmission bundleOrigins at: 4) port name = #port4. 
	self assert: transmission origins size = 4. 
	transmission origins 
		do: [:each | self assert: (each isKindOf: GLMSimplePort)]. 
	self assert: (transmission origins at: 1) name = #port1. 
	self assert: (transmission origins at: 2) name = #port2. 
	self assert: (transmission origins at: 3) name = #port3. 
	self assert: (transmission origins at: 4) name = #port4. 
	self assert: transmission activeOrigins size = 2. 
	transmission activeOrigins 
		do: [:each | self assert: (each isKindOf: GLMSimplePort)]. 
	self assert: (transmission activeOrigins at: 1) name = #port1. 
	self assert: (transmission activeOrigins at: 2) name = #port3. 
	self assert: transmission passiveOrigins size = 2. 
	transmission passiveOrigins 
		do: [:each | self assert: (each isKindOf: GLMSimplePort)]. 
	self assert: (transmission passiveOrigins at: 1) name = #port2. 
	self assert: (transmission passiveOrigins at: 2) name = #port4. 
	self assert: (transmission destination isKindOf: GLMSimplePort). 
	self assert: transmission destination name = #port5! !

!GLMBundleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPresentations 
	"Tests that the presentations are set up correctly." 
	 
	| transmission presentation1 presentation2 presentation3 | 
	transmission := GLMBundleTransmission new. 
	transmission 
		addActiveOrigin: 
			((GLMSimplePort new) 
				name: #port1; 
				value: 1). 
	transmission 
		addPassiveOrigin: 
			((GLMSimplePort new) 
				name: #port2; 
				value: 2). 
	transmission destination: (GLMSimplePort new name: #port3). 
	presentation1 := GLMPresentation new title: 'presentation1'. 
	presentation2 := GLMPresentation new title: 'presentation2'. 
	presentation3 := GLMPresentation new title: 'presentation3'. 
	transmission addPresentation: presentation1. 
	transmission addPresentation: presentation2. 
	transmission addPresentation: presentation3.	"Test correct setup" 
	self assert: transmission presentations size = 3. 
	self assert: (transmission presentations at: 1) == presentation1. 
	self assert: (transmission presentations at: 2) == presentation2. 
	self assert: (transmission presentations at: 3) == presentation3! !

!GLMBundleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTransmitting 
	"Tests that the origins are set up properly and that their order is retained." 
	 
	| transmission | 
	transmission := GLMBundleTransmission new. 
	transmission 
		addActiveOrigin: 
			((GLMSimplePort new) 
				name: #port1; 
				value: 1). 
	transmission destination: (GLMSimplePort new name: #port5). 
	self assert: transmission value = 1. 
	self assert: transmission destination value isNil. 
	transmission transmit. 
	self assert: transmission destination value = 1. 
	transmission 
		addPassiveOrigin: 
			((GLMSimplePort new) 
				name: #port2; 
				value: 2). 
	transmission 
		addActiveOrigin: 
			((GLMSimplePort new) 
				name: #port3; 
				value: 3). 
	transmission 
		addPassiveOrigin: 
			((GLMSimplePort new) 
				name: #port4; 
				value: 4). 
	self assert: transmission value = (GLMMultiValue withAll: #(1 2 3 4)). 
	transmission transmit. 
	self 
		assert: 
			transmission destination value = (GLMMultiValue withAll: #(1 2 3 4))! !

!GLMBundleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTriggerConditions 
	"Tests that the transmission is triggered only when the active origins change." 
	 
	| transmission port1 port2 | 
	transmission := GLMBundleTransmission new. 
	port1 := GLMPort new name: #port1. 
	port2 := GLMPort new name: #port2. 
	transmission addActiveOrigin: port1. 
	transmission addPassiveOrigin: port2. 
	transmission destination: (GLMPort new name: #port3). 
	self assert: (transmission originatesAt: port1). 
	self assert: (transmission originatesAt: port2) not! !

!GLMBundleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTriggerConditionsWithParentPort 
	 
	| transmission presentation | 
	transmission := GLMBundleTransmission new. 
	presentation := GLMPresentation new. 
	transmission 
		addActiveOrigin: 
			((GLMParentPort new) 
				presentation: presentation; 
				name: #port1; 
				yourself). 
	self 
		assert: 
			(transmission 
				originatesAt: 
					((GLMParentPort new) 
						presentation: presentation; 
						name: #port1; 
						yourself)). 
	self assert: (transmission originatesAt: (presentation pane port: #port1))! !

TestCase subclass: #GLMExplicitBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testJumpstartTransmission 
	 
	| browser presentation | 
	browser := GLMExplicitBrowser new. 
	presentation := GLMPresentation new. 
	browser addPane: GLMPane new. 
	browser 
		addJumpstart: 
			((GLMBundleTransmission new) 
				destination: (browser panes first port: #entity); 
				addPresentation: presentation; 
				yourself). 
	self assert: (browser panes first port: #entity) value isNil. 
	self assert: browser panes first presentations isEmpty. 
	(browser pane port: #entity) value: #foo. 
	self assert: (browser panes first port: #entity) value = #foo. 
	self assert: browser panes first presentations size = 1. 
	self assert: browser panes first presentations first == presentation! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testParentPort 
	 
	| browser outerPort1 outerPort2 | 
	browser := GLMExplicitBrowser new. 
	browser addPane: GLMTestPane new. 
	GLMTestPane new addPresentation: browser. 
	outerPort1 := (GLMParentPort new) 
		presentation: browser; 
		name: #outerPort1; 
		yourself. 
	outerPort2 := (GLMParentPort new) 
		presentation: browser; 
		name: #outerPort2; 
		yourself.	"Capturing" 
	browser 
		addTransmission: 
			((GLMSimpleTransmission new) 
				origin: outerPort1; 
				destination: (browser panes first port: #innerPort1); 
				yourself).	"Publishing" 
	browser 
		addTransmission: 
			((GLMSimpleTransmission new) 
				origin: (browser panes first port: #innerPort2); 
				destination: outerPort2; 
				yourself). 
	(browser pane port: #outerPort1) value: 42. 
	self assert: (browser panes first port: #innerPort1) value = 42. 
	self assert: (browser pane port: #outerPort1) seenTransmissions size = 1. 
	self 
		assert: 
			((browser pane port: #outerPort1) seenTransmissions first isKindOf: 
					GLMSimpleTransmission). 
	self 
		assert: 
			((browser pane port: #outerPort1) seenTransmissions first origin isKindOf: 
					GLMSimplePort). 
	self 
		assert: 
			(browser pane port: #outerPort1) seenTransmissions first destination 
				== (browser pane port: #outerPort1). 
	self 
		assert: 
			(browser panes first port: #innerPort1) seenTransmissions size = 1. 
	self 
		assert: 
			(browser panes first port: #innerPort1) seenTransmissions first 
				== browser transmissions first. 
	(browser panes first port: #innerPort2) value: 43. 
	self assert: (browser pane port: #outerPort2) value = 43. 
	self 
		assert: 
			(browser panes first port: #innerPort2) seenTransmissions size = 1. 
	self 
		assert: 
			((browser panes first port: #innerPort2) seenTransmissions first isKindOf: 
					GLMSimpleTransmission). 
	self 
		assert: 
			((browser panes first port: #innerPort2) seenTransmissions first origin 
				isKindOf: GLMSimplePort). 
	self 
		assert: 
			(browser panes first port: #innerPort2) seenTransmissions first destination 
				== (browser panes first port: #innerPort2). 
	self assert: (browser pane port: #outerPort2) seenTransmissions size = 1. 
	self 
		assert: 
			(browser pane port: #outerPort2) seenTransmissions first 
				== browser transmissions last! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPortLoopPublish 
	 
	| browser pane | 
	self signalFailure: 'The following test loops indefinetely.'. 
	browser := GLMExplicitBrowser new. 
	pane := GLMPane new. 
	browser addPane: pane. 
	browser 
		addPush: (pane port: #port) 
		as: #publishedPort. 
	browser 
		addPull: #publishedPort 
		to: (pane port: #port). 
	(pane port: #port) value: #foo. 
	self assert: browser pane ports size = 1. 
	self assert: (browser pane port: #publishedPort) value = #foo. 
	self assert: pane ports size = 1. 
	self assert: (pane port: #port) value = #foo! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPortPulling 
	 
	| browser pane | 
	browser := GLMExplicitBrowser new. 
	pane := GLMPane new. 
	browser addPane: pane. 
	browser 
		addPull: #port 
		to: (pane port: #pulledPort). 
	self assert: browser pulledPorts size = 1. 
	self 
		assert: browser pulledPorts first innerPort = (pane port: #pulledPort). 
	self assert: browser pulledPorts first outerPortName = #port. 
	self assert: (pane port: #pulledPort) value isNil. 
	(browser pane port: #port) value: #foo. 
	self assert: pane ports size = 1. 
	self assert: (pane port: #pulledPort) value = #foo! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPortPushing 
	 
	| browser pane | 
	browser := GLMExplicitBrowser new. 
	pane := GLMPane new. 
	browser addPane: pane. 
	browser 
		addPush: (pane port: #port) 
		as: #pushedPort. 
	self assert: browser pushedPorts size = 1. 
	self assert: browser pushedPorts first innerPort = (pane port: #port). 
	self assert: browser pushedPorts first outerPortName = #pushedPort. 
	self assert: (browser pane port: #pushedPort) value isNil. 
	(pane port: #port) value: #foo. 
	self assert: (browser pane port: #pushedPort) value = #foo! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSetup 
	 
	| browser pane1 pane2 transmission | 
	browser := GLMExplicitBrowser new. 
	pane1 := GLMTestPane new. 
	pane2 := GLMTestPane new. 
	transmission := (GLMSimpleTransmission new) 
		origin: (pane1 port: #selection); 
		destination: (pane2 port: #entity); 
		yourself. 
	browser addPane: pane1. 
	browser addPane: pane2. 
	browser addTransmission: transmission. 
	self assert: browser panes size = 2. 
	self assert: browser transmissions size = 1. 
	self assert: browser panes first == pane1. 
	self assert: browser panes last == pane2. 
	self assert: pane1 browser == browser. 
	self assert: pane2 browser == browser. 
	self assert: browser transmissions first == transmission! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimpleTransmission 
	 
	| browser pane1 pane2 presentation1 presentation2 transmission announcement | 
	browser := GLMExplicitBrowser new. 
	presentation1 := GLMPresentation new. 
	presentation2 := GLMPresentation new. 
	pane1 := GLMTestPane new. 
	pane2 := GLMTestPane new. 
	pane1 addPresentation: presentation1. 
	pane2 addPresentation: presentation2. 
	transmission := (GLMSimpleTransmission new) 
		origin: (pane1 port: #selection); 
		destination: (pane2 port: #entity); 
		yourself. 
	browser addPane: pane1. 
	browser addPane: pane2. 
	browser addTransmission: transmission. 
	announcement := nil. 
	presentation2 
		when: GLMContextChanged 
		do: [:ann | announcement := ann]. 
	presentation1 selection: 'foo'. 
	self assert: (pane1 port: #selection) seenTransmissions size = 1. 
	self assert: (pane2 port: #entity) seenTransmissions size = 1. 
	self 
		assert: (pane1 port: #selection) seenTransmissions first ~~ transmission. 
	self assert: (pane2 port: #entity) seenTransmissions first == transmission. 
	self assert: (pane2 port: #entity) value = 'foo'. 
	self assert: announcement notNil! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTransmissionBundle 
	 
	| browser pane1 pane2 pane3 presentation1 presentation2 transmission | 
	browser := GLMExplicitBrowser new. 
	presentation1 := GLMPresentation new. 
	presentation2 := GLMPresentation new. 
	pane1 := GLMTestPane new. 
	pane2 := GLMTestPane new. 
	pane3 := GLMTestPane new. 
	pane1 addPresentation: GLMPresentation new. 
	pane2 addPresentation: GLMPresentation new. 
	pane3 addPresentation: presentation1. 
	transmission := (GLMBundleTransmission new) 
		addActiveOrigin: (pane1 port: #selection); 
		addPassiveOrigin: (pane2 port: #selection); 
		destination: (pane3 port: #entity); 
		addPresentation: presentation2; 
		yourself. 
	browser addPane: pane1. 
	browser addPane: pane2. 
	browser addPane: pane3. 
	browser addTransmission: transmission. 
	self assert: pane3 presentations size = 1. 
	self assert: pane3 presentations first == presentation1.	"The first should not trigger any transmission (passive) 
	but the second should." 
	pane2 presentations first selection: #foo. 
	pane1 presentations first selection: #bar. 
	self assert: (pane1 port: #selection) seenTransmissions size = 1. 
	self assert: (pane2 port: #selection) seenTransmissions size = 1. 
	self 
		assert: (pane1 port: #selection) seenTransmissions first ~~ transmission. 
	self 
		assert: (pane2 port: #selection) seenTransmissions first ~~ transmission. 
	self assert: (pane3 port: #entity) seenTransmissions size = 2. 
	self assert: (pane3 port: #entity) seenTransmissions first ~~ transmission. 
	self assert: (pane3 port: #entity) seenTransmissions first value = nil. 
	self assert: (pane3 port: #entity) seenTransmissions last == transmission. 
	self 
		assert: 
			(pane3 port: #entity) value = (GLMMultiValue withAll: #(#bar #foo)). 
	self assert: pane3 presentations size = 1. 
	self assert: pane3 presentations first == presentation2! !

!GLMExplicitBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTriggeringConditions 
	 
	| browser pane1 pane2 transmission | 
	browser := GLMExplicitBrowser new. 
	browser pane: GLMTestPane new. 
	pane1 := GLMTestPane new. 
	pane2 := GLMTestPane new. 
	transmission := (GLMSimpleTransmission new) 
		origin: (pane1 port: #selection); 
		destination: (pane2 port: #entity); 
		yourself. 
	browser addPane: pane1. 
	browser addPane: pane2. 
	browser addTransmission: transmission. 
	browser 
		addPush: (pane1 port: #selection) 
		as: #pane1Selection. 
	self assert: (pane1 port: #selection) seenTransmissions isEmpty. 
	self assert: (pane2 port: #entity) seenTransmissions isEmpty. 
	self 
		assert: (browser pane port: #pane1Selection) seenTransmissions isEmpty. 
	(pane1 port: #selection) value: 12. 
	self assert: (pane1 port: #selection) seenTransmissions size = 1. 
	self 
		assert: 
			((pane1 port: #selection) seenTransmissions first isKindOf: 
					GLMSimpleTransmission). 
	self assert: (pane2 port: #entity) seenTransmissions size = 1. 
	self assert: (pane2 port: #entity) seenTransmissions first == transmission. 
	self 
		assert: (browser pane port: #pane1Selection) seenTransmissions size = 1. 
	self 
		assert: 
			((browser pane port: #pane1Selection) seenTransmissions first isKindOf: 
					GLMSimpleTransmission).	"Setting the same value should re-publish but not trigger inner transmissions." 
	(pane1 port: #selection) value: 12. 
	self assert: (pane1 port: #selection) seenTransmissions size = 2. 
	self 
		assert: 
			((pane1 port: #selection) seenTransmissions first isKindOf: 
					GLMSimpleTransmission). 
	self 
		assert: 
			((pane1 port: #selection) seenTransmissions last isKindOf: 
					GLMSimpleTransmission). 
	self assert: (pane2 port: #entity) seenTransmissions size = 1. 
	self assert: (pane2 port: #entity) seenTransmissions first == transmission. 
	self 
		assert: (browser pane port: #pane1Selection) seenTransmissions size = 2. 
	self 
		assert: 
			((browser pane port: #pane1Selection) seenTransmissions first isKindOf: 
					GLMSimpleTransmission). 
	self 
		assert: 
			((browser pane port: #pane1Selection) seenTransmissions first isKindOf: 
					GLMSimpleTransmission)! !

TestCase subclass: #GLMFinderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMFinderTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPaneCreation 
	 
	| browser | 
	browser := GLMFinder new. 
	browser addPresentation: (GLMPresentation new title: 'presentation1'). 
	self assert: browser panes isEmpty.	"Outer entity changed." 
	(browser pane port: #entity) value: #value1. 
	self assert: browser panes size = 1. 
	self assert: browser panes first browser == browser. 
	self assert: (browser panes first port: #entity) value = #value1. 
	self assert: browser panes first presentations size = 1. 
	self 
		assert: browser panes first presentations first title = 'presentation1'. 
	self 
		assert: 
			browser panes first presentations first ~~ browser presentations first.	"First pane selection changed." 
	(browser panes first port: #selection) value: #value2. 
	self assert: browser panes size = 2. 
	self assert: browser panes last browser == browser. 
	self assert: (browser panes last port: #entity) value = #value2. 
	self 
		assert: browser panes last presentations first title = 'presentation1'. 
	self 
		assert: 
			browser panes last presentations first ~~ browser presentations first. 
	self 
		assert: 
			browser panes last presentations first 
				~~ browser panes first presentations first.	"Second pane selection changed." 
	(browser panes last port: #selection) value: #value3. 
	self assert: browser panes size = 3. 
	self assert: browser panes last browser == browser. 
	self assert: (browser panes last port: #entity) value = #value3.	"First pane selection changed again." 
	(browser panes first port: #selection) value: #value4. 
	self assert: browser panes size = 2. 
	self assert: browser panes last browser == browser. 
	self assert: (browser panes last port: #entity) value = #value4.	"Outer entity changed again." 
	(browser pane port: #entity) value: #value5. 
	self assert: browser panes size = 1. 
	self assert: browser panes last browser == browser. 
	self assert: (browser panes last port: #entity) value = #value5! !

!GLMFinderTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPortPublishing 
	 
	| browser | 
	browser := GLMFinder new. 
	(browser pane port: #entity) value: #value1. 
	self assert: browser panes size = 1. 
	self assert: browser pane ports size = 1. 
	self assert: (browser pane port: #entity) value = #value1. 
	(browser panes first port: #selection) value: #value2. 
	self assert: browser panes size = 2. 
	self assert: browser pane ports size = 2. 
	self assert: (browser pane port: #entity) value = #value1. 
	self assert: (browser pane port: #selection) value = #value2. 
	(browser panes last port: #selection) value: #value3. 
	self assert: browser panes size = 3. 
	self assert: browser pane ports size = 2. 
	self assert: (browser pane port: #entity) value = #value1. 
	self assert: (browser pane port: #selection) value = #value3. 
	(browser panes first port: #hover) value: #value4. 
	self assert: browser panes size = 3. 
	self assert: browser pane ports size = 3. 
	self assert: (browser pane port: #entity) value = #value1. 
	self assert: (browser pane port: #selection) value = #value3. 
	self assert: (browser pane port: #hover) value = #value4! !

TestCase subclass: #GLMNestedBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMNestedBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testChangeInnerSelection 
	"Tests setting the selection of an inner pane from the oustide. 
 
	The test sets up a browser with two panes. The first contains a 
	list of numbers, the second a browser which again contains one 
	pane with a list of the same numbers. When the selection in the 
	first pane changes, we want the same number to be selected in 
	the second pane. 
 
	To achieve this, we pull in a port on the second pane and copy 
	it to the inner pane using a simple transmission." 
	 
	| browser1 browser2 list1 list2 | 
	browser1 := GLMExplicitBrowser new. 
	browser2 := GLMExplicitBrowser new. 
	list1 := GLMListPresentation new. 
	list2 := GLMListPresentation new. 
	browser1 addPane: GLMPane new. 
	browser1 addPane: GLMPane new. 
	browser2 addPane: GLMPane new. 
	browser1 panes first addPresentation: list1. 
	browser1 panes last addPresentation: browser2. 
	browser2 panes first addPresentation: list2. 
	browser1 
		addTransmission: 
			((GLMSimpleTransmission new) 
				origin: (browser1 panes first port: #selection); 
				destination: (browser1 panes last port: #innerSelection); 
				yourself). 
	browser2 
		addTransmission: 
			((GLMSimpleTransmission new) 
				origin: 
						((GLMParentPort new) 
								presentation: browser2; 
								name: #innerSelection; 
								yourself); 
				destination: (browser2 panes first port: #selection); 
				yourself). 
	list1 entity: #(1 2 3). 
	list2 entity: #(1 2 3). 
	list1 selection: 1. 
	self assert: list2 selection = 1. 
	list1 selection: 2. 
	self assert: list2 selection = 2! !

!GLMNestedBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testEntityPropogation 
	"This tests the order in which presentations and values are updated. 
 
	This is of importance as, in this test, we add a browser to a pane when a 
	certain port of that pane gets updated using a BundleTransmission but this 
	browser has a transmission from its #outer pane that refers to the same 
	port.  In such a case we want the presentation to be set first so that it can 
	be notified when the outer port changes." 
	 
	| browser1 browser2 presentation | 
	browser1 := GLMExplicitBrowser new. 
	browser2 := GLMExplicitBrowser new. 
	presentation := GLMPresentation new. 
	browser1 addPane: GLMPane new. 
	browser2 addPane: GLMPane new.	"Jumpstart presentation for outer browser." 
	browser1 
		addTransmission: 
			((GLMBundleTransmission new) 
				addActiveOrigin: 
						((GLMParentPort new) 
								presentation: browser1; 
								name: #entity; 
								yourself); 
				destination: (browser1 panes first port: #entity); 
				addPresentation: browser2; 
				yourself).	"Jumpstart presentation for inner browser." 
	browser2 
		addTransmission: 
			((GLMBundleTransmission new) 
				addActiveOrigin: 
						((GLMParentPort new) 
								presentation: browser2; 
								name: #entity; 
								yourself); 
				destination: (browser2 panes first port: #entity); 
				addPresentation: presentation; 
				yourself). 
	self assert: (browser1 pane port: #entity) value isNil. 
	self assert: browser1 panes size = 1. 
	self assert: (browser1 panes first port: #entity) value isNil. 
	self assert: browser1 panes first presentations isEmpty. 
	self assert: browser2 panes size = 1. 
	self assert: (browser2 panes first port: #entity) value isNil. 
	self assert: browser2 panes first presentations isEmpty. 
	(browser1 pane port: #entity) value: 42. 
	self assert: (browser1 pane port: #entity) value = 42. 
	self assert: browser1 panes size = 1. 
	self assert: (browser1 panes first port: #entity) value = 42. 
	self assert: browser1 panes first presentations size = 1. 
	self assert: browser1 panes first presentations first == browser2. 
	self assert: browser2 panes size = 1. 
	self assert: (browser2 panes first port: #entity) value = 42. 
	self assert: browser2 panes first presentations size = 1. 
	self assert: browser2 panes first presentations first == presentation! !

!GLMNestedBrowserTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testEntityPropogationAnnouncements 
	"This tests the order in which announcements are raised in nested browsers. 
 
	This is of importance as, in this test, we add a browser to a pane when a 
	certain port of that pane gets updated using a BundleTransmission but this 
	browser has a transmission from its #outer pane that refers to the same 
	port.  In such a case we want the presentation to be set first so that it can 
	be notified when the outer port changes. 
 
	This is a sister test to testEntityPropogation that ensures that the 
	announcements are raised in the right order and not redundantly." 
	 
	| browser1 browser2 presentation announcements | 
	browser1 := GLMExplicitBrowser new. 
	browser2 := GLMExplicitBrowser new. 
	presentation := GLMPresentation new. 
	browser1 pane. 
	browser1 addPane: GLMPane new. 
	browser2 addPane: GLMPane new. 
	announcements := OrderedCollection new. 
	self assert: browser1 pane presentations size = 1. 
	self assert: browser1 pane presentations first == browser1. 
	browser1 whenContextChanged: [:ann | announcements addLast: ann]. 
	browser2 whenContextChanged: [:ann | announcements addLast: ann]. 
	presentation whenContextChanged: [:ann | announcements addLast: ann]. 
	browser1 pane 
		whenPresentationsChanged: [:ann | announcements addLast: ann]. 
	browser1 panes first 
		whenPresentationsChanged: [:ann | announcements addLast: ann]. 
	browser2 panes first 
		whenPresentationsChanged: [:ann | announcements addLast: ann]. 
	browser1 pane 
		whenMatchingPresentationsChanged: [:ann | announcements addLast: ann]. 
	browser1 panes first 
		whenMatchingPresentationsChanged: [:ann | announcements addLast: ann]. 
	browser2 panes first 
		whenMatchingPresentationsChanged: [:ann | announcements addLast: ann].	"Jumpstart presentation for outer browser." 
	browser1 
		addTransmission: 
			((GLMBundleTransmission new) 
				addActiveOrigin: 
						((GLMParentPort new) 
								presentation: browser1; 
								name: #entity; 
								yourself); 
				destination: (browser1 panes first port: #entity); 
				addPresentation: browser2; 
				yourself).	"Jumpstart presentation for inner browser." 
	browser2 
		addTransmission: 
			((GLMBundleTransmission new) 
				addActiveOrigin: 
						((GLMParentPort new) 
								presentation: browser2; 
								name: #entity; 
								yourself); 
				destination: (browser2 panes first port: #entity); 
				addPresentation: presentation; 
				yourself). 
	self assert: announcements isEmpty. 
	(browser1 pane port: #entity) value: 42. 
	self assert: announcements size = 8.	"1. The outer browser's outer pane changes its value." 
	self assert: ((announcements at: 1) isKindOf: GLMContextChanged). 
	self assert: (announcements at: 1) presentation == browser1. 
	self assert: (announcements at: 1) property = #entity. 
	self assert: (announcements at: 1) value = 42.	"2. The jumpstart transmission of the outer browser is triggered, updating the presentations..." 
	self assert: ((announcements at: 2) isKindOf: GLMPresentationsChanged). 
	self assert: (announcements at: 2) pane == browser1 panes first. 
	self assert: (announcements at: 2) oldPresentations isEmpty. 
	self assert: (announcements at: 2) presentations size = 1. 
	self assert: (announcements at: 2) presentations first == browser2.	"3. ...and then the context." 
	self assert: ((announcements at: 3) isKindOf: GLMContextChanged). 
	self assert: (announcements at: 3) presentation == browser2. 
	self assert: (announcements at: 3) property = #entity. 
	self assert: (announcements at: 3) value = 42.	"4. The jumpstart transmission of the inner browser is triggered, updating the presentations..." 
	self assert: ((announcements at: 4) isKindOf: GLMPresentationsChanged). 
	self assert: (announcements at: 4) pane == browser2 panes first. 
	self assert: (announcements at: 4) oldPresentations isEmpty. 
	self assert: (announcements at: 4) presentations size = 1. 
	self assert: (announcements at: 4) presentations first == presentation.	"5. ...and then the context." 
	self assert: ((announcements at: 5) isKindOf: GLMContextChanged). 
	self assert: (announcements at: 5) presentation == presentation. 
	self assert: (announcements at: 5) property = #entity. 
	self assert: (announcements at: 5) value = 42.	"6. Due to the context change of 5, the innermost presentation now matches." 
	self 
		assert: ((announcements at: 6) isKindOf: GLMMatchingPresentationsChanged). 
	self assert: (announcements at: 6) pane == browser2 panes first. 
	self assert: (announcements at: 6) oldMatchingPresentations isEmpty. 
	self assert: (announcements at: 6) matchingPresentations size = 1. 
	self 
		assert: (announcements at: 6) matchingPresentations first == presentation.	"7. Due to the context change of 3, the inner browser now matches." 
	self 
		assert: ((announcements at: 7) isKindOf: GLMMatchingPresentationsChanged). 
	self assert: (announcements at: 7) pane == browser1 panes first. 
	self assert: (announcements at: 7) oldMatchingPresentations isEmpty. 
	self assert: (announcements at: 7) matchingPresentations size = 1. 
	self assert: (announcements at: 7) matchingPresentations first == browser2.	"8. Due to the context change of 1, the outer browser now matches." 
	self 
		assert: ((announcements at: 8) isKindOf: GLMMatchingPresentationsChanged). 
	self assert: (announcements at: 8) pane == browser1 pane. 
	self assert: (announcements at: 8) oldMatchingPresentations isEmpty. 
	self assert: (announcements at: 8) matchingPresentations size = 1. 
	self assert: (announcements at: 8) matchingPresentations first == browser1! !

TestCase subclass: #GLMPaneTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMPaneTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPorts 
	 
	| pane port | 
	pane := GLMPane new. 
	self assert: pane ports isEmpty. 
	port := pane port: #foo. 
	self assert: pane ports size = 1. 
	self assert: pane ports first == port. 
	self assert: (pane port: #foo) == port! !

!GLMPaneTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPresentationConditions 
	 
	| pane presentation1 presentation2 presentation3 | 
	pane := GLMPane new. 
	presentation1 := GLMPresentation new 
		condition: [:first :second | first = 1 and: [second = 2]]. 
	presentation2 := GLMPresentation new condition: [:first | first = #foo]. 
	presentation3 := GLMPresentation new. 
	pane addPresentation: presentation1. 
	pane addPresentation: presentation2. 
	pane addPresentation: presentation3. 
	(pane port: #entity) value: (GLMMultiValue withAll: #(1 2)). 
	self assert: pane matchingPresentations size = 2. 
	self assert: pane matchingPresentations first == presentation1. 
	self assert: pane matchingPresentations last == presentation3. 
	(pane port: #entity) value: (GLMMultiValue withAll: #(#foo 2)). 
	self assert: pane matchingPresentations size = 2. 
	self assert: pane matchingPresentations first == presentation2. 
	self assert: pane matchingPresentations last == presentation3. 
	(pane port: #entity) value: (GLMMultiValue withAll: #(nil 2)). 
	self assert: pane matchingPresentations size = 1! !

!GLMPaneTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPresentations 
	 
	| pane presentation announcement | 
	pane := GLMPane new. 
	presentation := GLMPresentation new. 
	self assert: pane presentations isEmpty. 
	announcement := nil. 
	pane whenPresentationsChanged: [:ann | announcement := ann]. 
	pane addPresentation: presentation. 
	self assert: pane presentations size = 1. 
	self assert: pane presentations first == presentation. 
	self assert: presentation pane == pane. 
	self assert: announcement notNil! !

!GLMPaneTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimplePresentations 
	 
	| pane presentation announcement | 
	pane := GLMPane new. 
	presentation := GLMPresentation new. 
	self assert: pane presentations isEmpty. 
	announcement := nil. 
	pane whenPresentationsChanged: [:ann | announcement := ann]. 
	pane addPresentation: presentation. 
	self assert: pane presentations size = 1. 
	self assert: pane presentations first == presentation. 
	self assert: presentation pane == pane. 
	self assert: announcement notNil! !

TestCase subclass: #GLMPortTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMPortTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPanePort 
	 
	| port | 
	port := GLMTestPane new port: #port1. 
	self assert: port seenTransmissions isEmpty. 
	port value: 42. 
	self assert: port seenTransmissions size = 1. 
	self assert: port seenTransmissions first origin value = 42. 
	self assert: port seenTransmissions first origin ~= port. 
	self assert: port seenTransmissions first destination == port. 
	self assert: port value = 42! !

!GLMPortTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testParentPort 
	 
	| presentation port | 
	presentation := GLMPresentation new. 
	port := (GLMParentPort new) 
		name: #port; 
		presentation: presentation. 
	port value: 42. 
	self assert: (presentation pane port: #port) value = 42. 
	(presentation pane port: #port) value: 43. 
	self assert: port value = 43! !

!GLMPortTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPortEquality 
	 
	| presentation port | 
	presentation := GLMPresentation new. 
	port := (GLMParentPort new) 
		name: #port; 
		presentation: presentation. 
	self assert: port = (presentation pane port: #port). 
	self assert: (presentation pane port: #port) = port. 
	self assert: port hash = (presentation pane port: #port) hash. 
	self assert: (presentation pane port: #port) hash = port hash! !

!GLMPortTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimplePort 
	 
	| port | 
	port := GLMSimplePort new. 
	port name: #foo. 
	port value: 42. 
	self assert: port name = #foo. 
	self assert: port value = 42. 
	self assert: port hasPane not! !

TestCase subclass: #GLMPresentationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMPresentationTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testAnnouncement 
	 
	| pane presentation announcement | 
	pane := GLMPane new. 
	presentation := GLMPresentation new. 
	pane addPresentation: presentation. 
	announcement := nil. 
	presentation 
		when: GLMContextChanged 
		do: [:ann | announcement := ann]. 
	(pane port: #entity) value: 42. 
	self assert: announcement notNil. 
	self assert: announcement property = #entity. 
	self assert: announcement value = 42! !

!GLMPresentationTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testContext 
	 
	| pane presentation | 
	pane := GLMPane new. 
	presentation := GLMPresentation new. 
	pane addPresentation: presentation. 
	presentation selection: 42. 
	self assert: (pane port: #selection) value = 42. 
	(pane port: #entity) value: 'foo'. 
	self assert: presentation entity = 'foo'! !

!GLMPresentationTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testDisplay 
	 
	| presentation | 
	presentation := GLMPresentation new. 
	presentation entity: #foo. 
	self assert: presentation displayValue = #foo. 
	presentation entity: #(#foo #bar). 
	self assert: presentation displayValue = #(#foo #bar). 
	presentation entity: (GLMMultiValue withAll: #(#foo #bar)). 
	self assert: presentation displayValue = #foo. 
	presentation transformation: [:first :second | (first , second) asSymbol]. 
	self assert: presentation displayValue = #foobar! !

!GLMPresentationTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testMatching 
	 
	| presentation | 
	presentation := GLMPresentation new. 
	self assert: presentation usesImplicitNotNil. 
	(presentation pane port: #entity) value: nil. 
	self assert: presentation matches not. 
	(presentation pane port: #entity) value: 1. 
	self assert: presentation matches. 
	presentation useExplicitNotNil. 
	self assert: presentation usesImplicitNotNil not. 
	(presentation pane port: #entity) value: nil. 
	self assert: presentation matches. 
	(presentation pane port: #entity) value: 1. 
	self assert: presentation matches. 
	presentation condition: [:x | x > 1]. 
	presentation useImplicitNotNil. 
	self assert: presentation usesImplicitNotNil. 
	(presentation pane port: #entity) value: nil. 
	self assert: presentation matches not. 
	(presentation pane port: #entity) value: 1. 
	self assert: presentation matches not. 
	(presentation pane port: #entity) value: 2. 
	self assert: presentation matches. 
	presentation useExplicitNotNil. 
	(presentation pane port: #entity) value: 1. 
	self assert: presentation matches not. 
	(presentation pane port: #entity) value: 2. 
	self assert: presentation matches. 
	self assert: presentation usesImplicitNotNil not. 
	[(presentation pane port: #entity) value: nil. 
	presentation matches. 
	self signalFailure: 'Should have raised error'] 
		on: MessageNotUnderstood 
		do: 
			[:err |  
			self assert: err message selector = #>. 
			self assert: err message arguments = (Array with: 1)]! !

!GLMPresentationTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSetup 
	 
	| presentation | 
	presentation := GLMPresentation new. 
	self assert: presentation pane notNil. 
	self assert: (presentation pane presentations includes: presentation)! !

TestCase subclass: #GLMScriptingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testConditionalPresentation 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two. 
	browser 
		showOn: #two; 
		from: #one; 
		using: 
				[browser text when: [:x | x = 42]. 
					browser list when: [:x | x > 42]]. 
	((browser paneNamed: #one) port: #selection) value: 42. 
	self assert: (browser paneNamed: #two) matchingPresentations size = 1. 
	self 
		assert: 
			((browser paneNamed: #two) matchingPresentations first isKindOf: 
					GLMTextPresentation). 
	((browser paneNamed: #one) port: #selection) value: 43. 
	self assert: (browser paneNamed: #two) matchingPresentations size = 1. 
	self 
		assert: 
			((browser paneNamed: #two) matchingPresentations first isKindOf: 
					GLMListPresentation)! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testNestedBrowser 
	 
	| browser innerBrowser | 
	browser := GLMTableLayoutBrowser new. 
	browser column: #one. 
	innerBrowser := GLMTableLayoutBrowser new. 
	innerBrowser column: #innerOne. 
	innerBrowser 
		showOn: #innerOne; 
		using: [innerBrowser text]. 
	browser 
		showOn: #one; 
		using: [browser custom: innerBrowser]. 
	browser startOn: 42. 
	self assert: ((browser paneNamed: #one) port: #entity) value = 42. 
	self 
		assert: ((innerBrowser paneNamed: #innerOne) port: #entity) value = 42! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testNestedBrowserWithExplicitTransmission 
	 
	| browser innerBrowser | 
	browser := GLMTableLayoutBrowser new. 
	browser column: #one. 
	innerBrowser := GLMTableLayoutBrowser new. 
	innerBrowser column: #innerOne. 
	innerBrowser 
		showOn: #innerOne; 
		from: #outer -> #entity; 
		using: [innerBrowser text]. 
	browser 
		showOn: #one; 
		using: [browser custom: innerBrowser]. 
	browser startOn: 42. 
	self assert: ((browser paneNamed: #one) port: #entity) value = 42. 
	self 
		assert: ((innerBrowser paneNamed: #innerOne) port: #entity) value = 42! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testPublishing 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser column: #classes. 
	browser 
		push: #classes -> #selection 
		as: #selectedClass. 
	browser 
		pull: #highlightedClass 
		to: #classes -> #highlight. 
	self assert: browser pushedPorts size = 1. 
	self 
		assert: 
			browser pushedPorts first innerPort 
				= ((browser paneNamed: #classes) port: #selection). 
	self assert: browser pushedPorts first outerPortName = #selectedClass. 
	self assert: browser pulledPorts size = 1. 
	self 
		assert: 
			browser pulledPorts first innerPort 
				= ((browser paneNamed: #classes) port: #highlight). 
	self assert: browser pulledPorts first outerPortName = #highlightedClass! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testReferences 
	 
	| browser pane1 | 
	browser := GLMExplicitBrowser new. 
	pane1 := GLMPane new name: #pane1. 
	browser addPane: pane1. 
	self 
		assert: (browser resolveOriginPort: #pane1) == (pane1 port: #selection). 
	self 
		assert: (browser resolveOriginPort: #pane1 -> #foo) == (pane1 port: #foo). 
	self 
		assert: (browser resolveDestinationPort: #pane1) == (pane1 port: #entity). 
	self 
		assert: 
			(browser resolveDestinationPort: #pane1 -> #foo) == (pane1 port: #foo). 
	self assert: ((browser resolveOriginPort: #outer) isKindOf: GLMParentPort). 
	self assert: (browser resolveOriginPort: #outer) presentation == browser. 
	self assert: (browser resolveOriginPort: #outer) name = #selection. 
	self 
		assert: 
			((browser resolveOriginPort: #outer -> #foo) isKindOf: GLMParentPort). 
	self 
		assert: 
			(browser resolveOriginPort: #outer -> #foo) presentation == browser. 
	self assert: (browser resolveOriginPort: #outer -> #foo) name = #foo. 
	self 
		assert: 
			((browser resolveDestinationPort: #outer) isKindOf: GLMParentPort). 
	self 
		assert: (browser resolveDestinationPort: #outer) presentation == browser. 
	self assert: (browser resolveDestinationPort: #outer) name = #entity. 
	self 
		assert: 
			((browser resolveDestinationPort: #outer -> #foo) isKindOf: GLMParentPort). 
	self 
		assert: 
			(browser resolveDestinationPort: #outer -> #foo) presentation == browser. 
	self assert: (browser resolveDestinationPort: #outer -> #foo) name = #foo! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testScriptingDefaults 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser startOn: 42. 
	self assert: browser panes size = 1. 
	self assert: browser jumpstartTransmissions size = 1. 
	self 
		assert: 
			browser jumpstartTransmissions first destination 
				= (browser panes first port: #entity)! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimpleTransmission 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two. 
	browser 
		sendTo: #two -> #port2 
		from: #one -> #port1. 
	self assert: browser transmissions size = 1. 
	self assert: (browser transmissions first isKindOf: GLMSimpleTransmission). 
	self 
		assert: 
			browser transmissions first origin 
				= ((browser paneNamed: #one) port: #port1). 
	self 
		assert: 
			browser transmissions first destination 
				= ((browser paneNamed: #two) port: #port2)! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testStartTransmission 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two. 
	browser 
		showOn: #one; 
		using: [browser list]. 
	self assert: browser transmissions isEmpty. 
	self assert: browser jumpstartTransmissions size = 1. 
	self assert: (browser paneNamed: #one) presentations isEmpty. 
	self assert: (browser paneNamed: #two) presentations isEmpty. 
	browser startOn: 15. 
	self assert: (browser paneNamed: #one) presentations size = 1. 
	self assert: (browser paneNamed: #two) presentations isEmpty! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testStartWithOnlyJumpStart 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two. 
	browser 
		showOn: #one; 
		using: [browser list]. 
	browser 
		showOn: #two; 
		from: #one; 
		using: [browser text]. 
	self assert: browser transmissions size = 1. 
	self assert: browser jumpstartTransmissions size = 1. 
	self assert: (browser paneNamed: #one) presentations isEmpty. 
	self assert: (browser paneNamed: #two) presentations isEmpty. 
	browser startOn: #(15 25 35). 
	self assert: (browser paneNamed: #one) presentations size = 1. 
	self assert: (browser paneNamed: #one) matchingPresentations size = 1. 
	self assert: (browser paneNamed: #two) presentations isEmpty. 
	self assert: (browser paneNamed: #two) matchingPresentations isEmpty! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testThreePanesTransmissions 
	 
	| browser one two | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two; 
		column: #three. 
	browser 
		showOn: #three; 
		from: #one; 
		passivelyFrom: #two; 
		using: 
				[browser text 
						display: 
							[:x :y |  
							one := x. 
							two := y]]. 
	self assert: browser jumpstartTransmissions isEmpty. 
	self assert: browser transmissions size = 1. 
	self assert: browser transmissions first passiveOrigins size = 1. 
	self assert: browser transmissions first activeOrigins size = 1. 
	((browser paneNamed: #two) port: #selection) value: 15. 
	self assert: ((browser paneNamed: #two) port: #selection) value = 15. 
	self assert: ((browser paneNamed: #three) port: #entity) value isNil. 
	((browser paneNamed: #one) port: #selection) value: 20. 
	self assert: ((browser paneNamed: #one) port: #selection) value = 20. 
	self assert: ((browser paneNamed: #three) port: #entity) value first = 20. 
	self assert: ((browser paneNamed: #three) port: #entity) value last = 15! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTransmissionWithTwoOrigins 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two; 
		column: #three. 
	browser 
		showOn: #three; 
		from: #one; 
		from: #two; 
		using: [browser text]. 
	((browser paneNamed: #one) port: #selection) value: 42. 
	self assert: ((browser paneNamed: #three) port: #entity) value first = 42. 
	self assert: ((browser paneNamed: #three) port: #entity) value last = nil. 
	self assert: (browser paneNamed: #three) presentations size = 1. 
	self assert: (browser paneNamed: #three) matchingPresentations size = 1. 
	((browser paneNamed: #two) port: #selection) value: 43. 
	self assert: ((browser paneNamed: #three) port: #entity) value first = 42. 
	self assert: ((browser paneNamed: #three) port: #entity) value last = 43. 
	self assert: (browser paneNamed: #three) matchingPresentations size = 1! !

!GLMScriptingTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTwoTransmissionsWithOnlyOneEvent 
	 
	| browser | 
	browser := GLMTableLayoutBrowser new. 
	browser 
		column: #one; 
		column: #two; 
		column: #three. 
	browser 
		showOn: #two; 
		from: #one; 
		using: [browser list]. 
	browser 
		showOn: #three; 
		from: #two; 
		using: [browser text]. 
	self assert: browser transmissions size = 2. 
	self assert: browser jumpstartTransmissions size = 0. 
	self assert: (browser paneNamed: #one) presentations isEmpty. 
	self assert: (browser paneNamed: #two) presentations isEmpty. 
	self assert: (browser paneNamed: #three) presentations isEmpty. 
	((browser paneNamed: #one) port: #selection) value: 42. 
	self assert: ((browser paneNamed: #two) port: #entity) value = 42. 
	self assert: (browser paneNamed: #one) presentations isEmpty. 
	self assert: (browser paneNamed: #two) presentations size = 1. 
	self assert: (browser paneNamed: #two) matchingPresentations size = 1. 
	self assert: (browser paneNamed: #three) presentations isEmpty. 
	self assert: (browser paneNamed: #three) matchingPresentations isEmpty! !

TestCase subclass: #GLMSimpleTransmissionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMSimpleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimple 
	 
	| pane1 pane2 transmission | 
	pane1 := GLMPane new. 
	pane2 := GLMPane new. 
	transmission := (GLMSimpleTransmission new) 
		origin: (pane1 port: #output); 
		destination: (pane2 port: #input); 
		yourself. 
	(pane1 port: #output) value: 42. 
	self assert: (pane1 port: #output) value = 42. 
	self assert: (pane2 port: #input) value = nil. 
	transmission transmit. 
	self assert: (pane1 port: #output) value = 42. 
	self assert: (pane2 port: #input) value = 42! !

!GLMSimpleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTriggerConditions 
	"Tests that the transmission is triggered only when the origin changes." 
	 
	| transmission port1 port2 | 
	transmission := GLMSimpleTransmission new. 
	port1 := GLMPort new name: #port1. 
	port2 := GLMPort new name: #port2. 
	transmission origin: port1. 
	transmission destination: port2. 
	self assert: (transmission originatesAt: port1). 
	self assert: (transmission originatesAt: port2) not. 
	self assert: (transmission originatesAt: GLMPort new) not! !

!GLMSimpleTransmissionTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testTriggerConditionsWithParentPort 
	 
	| transmission presentation | 
	transmission := GLMSimpleTransmission new. 
	presentation := GLMPresentation new. 
	transmission 
		origin: 
			((GLMParentPort new) 
				presentation: presentation; 
				name: #port1; 
				yourself). 
	self 
		assert: 
			(transmission 
				originatesAt: 
					((GLMParentPort new) 
						presentation: presentation; 
						name: #port1; 
						yourself)). 
	self assert: (transmission originatesAt: (presentation pane port: #port1))! !

TestCase subclass: #GLMTableLayoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMTableLayoutTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testComplex 
	 
	| layout | 
	layout := GLMTableLayoutBrowser new. 
	layout 
		addColumn: 
			[:column |  
			column addRow: #second. 
			column addRow: #third. 
			column addRow: #fourth. 
			column 
				addRow: 
					[:row |  
					row addColumn: #fifth. 
					row addColumn: #sixth]]. 
	layout addColumn: #first. 
	self assert: layout cell children size = 2. 
	self assert: layout cell children first children size = 4. 
	self assert: layout cell children first children last children size = 2! !

!GLMTableLayoutTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testOneSimpleColumnAndOneNested 
	 
	| layout | 
	layout := GLMTableLayoutBrowser new. 
	layout 
		addColumn: 
			[:column |  
			column addRow: #second. 
			column addRow: #third. 
			column addRow: #fourth]. 
	layout addColumn: #first. 
	self assert: layout cell children size = 2. 
	self assert: layout cell children first children size = 3! !

!GLMTableLayoutTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testOneSimpleRowAndOneNested 
	 
	| layout | 
	layout := GLMTableLayoutBrowser new. 
	layout addRow: #first. 
	layout 
		addRow: 
			[:row |  
			row addColumn: #second. 
			row addColumn: #third. 
			row addColumn: #fourth]. 
	self assert: layout cell children size = 2. 
	self assert: layout cell children last children size = 3! !

!GLMTableLayoutTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimpleColumns 
	 
	| layout | 
	layout := GLMTableLayoutBrowser new. 
	layout addColumn: #first. 
	layout addColumn: #second. 
	self assert: layout cell children size = 2! !

!GLMTableLayoutTest methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testSimpleRows 
	 
	| layout | 
	layout := GLMTableLayoutBrowser new. 
	layout addRow: #first. 
	layout addRow: #second. 
	self assert: layout cell children size = 2. 
	self assert: layout cell isColumn. 
	self assert: layout cell children first isRow! !

TestCase subclass: #GLMTestExtensions
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMTestExtensions methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testGlamourValue 
	"Block closure" 
	 
	self assert: ([#foo] glamourValue: #(1 2)) = #foo. 
	self 
		assert: ([:first | Array with: first] glamourValue: #(1 2)) = #(#(1 2)). 
	self 
		assert: 
			([:first :second | Array with: first with: second] glamourValue: #(1 2)) = 
					#(#(1 2) nil). 
	self 
		assert: 
			([:first :second | Array with: first with: second] 
				glamourValue: (GLMMultiValue with: 1 with: 2)) = #(1 2).	"Symbol" 
	self assert: (#yourself glamourValueWithArgs: #(1 2)) = 1. 
	self assert: (#yourself glamourValueWithArgs: #()) = nil	"self assert: (#yourself glamourValue: (MultiValue with: 1 with: 2)) = 1."! !

!GLMTestExtensions methodsFor: 'tests' stamp: ' 25/2/09 23:39'!
testGlamourValueWithArgs 
	"Block closure" 
	 
	self assert: ([#foo] glamourValueWithArgs: #(1 2)) = #foo. 
	self 
		assert: 
			([:first | Array with: first] glamourValueWithArgs: #(1 2)) = #(1). 
	self 
		assert: 
			([:first :second | Array with: first with: second] glamourValueWithArgs: 
					#(1 2)) = #(1 2). 
	self 
		assert: 
			([:first :second :third | Array with: first with: second with: third] 
				glamourValueWithArgs: #(1 2)) = #(1 2 nil).	"Symbol" 
	self assert: (#yourself glamourValueWithArgs: #(1 2)) = 1. 
	self assert: (#yourself glamourValueWithArgs: #()) = nil! !

GLMPanePort subclass: #GLMTestPort
	instanceVariableNames: 'seenTransmissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Glamour-Tests'!

!GLMTestPort methodsFor: 'accessing' stamp: ' 25/2/09 23:39'!
receive: aTransmission 
	 
	self seenTransmissions addLast: aTransmission. 
	^super receive: aTransmission! !

!GLMTestPort methodsFor: 'accessing' stamp: ' 25/2/09 23:39'!
seenTransmissions 
	 
	^seenTransmissions ifNil: [seenTransmissions := OrderedCollection new]! !
